<?php

namespace Bixo\Offplan\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Offplan\Forms\Booking as BookingForm;
use Bixo\Offplan\Http\Requests\BookingResourceRequest;
use Bixo\Offplan\Http\Resources\BookingResource;
use Bixo\Offplan\Http\Resources\BookingsCollection;
use Bixo\Offplan\Models\Booking;
use Bixo\Offplan\Actions\BookingAction;
use Bixo\Offplan\Actions\BookingActions;


/**
 * Resource controller class for booking.
 */
class BookingResourceController extends BaseController
{

    /**
     * Initialize booking resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = BookingForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.offplan.modules'), 'offplan', guard_url('offplan'));
            return $next($request);
        });
    }

    /**
     * Display a list of booking.
     *
     * @return Response
     */
    public function index(BookingResourceRequest $request)
    {
        $request = $request->all();
        $page = BookingActions::run('paginate', $request);

        $data = new BookingsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('offplan::booking.names'))
            ->view('offplan::booking.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display booking.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function show(BookingResourceRequest $request, Booking $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BookingResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('offplan::booking.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('offplan::booking.show')
            ->output();
    }

    /**
     * Show the form for creating a new booking.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BookingResourceRequest $request, Booking $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BookingResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('offplan::booking.name'))
            ->view('offplan::booking.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new booking.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BookingResourceRequest $request, Booking $model)
    {
        try {
            $request = $request->all();
            $model = BookingAction::run('store', $model, $request);
            $data = new BookingResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('offplan::booking.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('offplan/booking/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/offplan/booking'))
                ->redirect();
        }

    }

    /**
     * Show booking for editing.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function edit(BookingResourceRequest $request, Booking $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new BookingResource($model);
        // return view('offplan::booking.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('offplan::booking.name'))
            ->view('offplan::booking.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the booking.
     *
     * @param Request $request
     * @param Model   $booking
     *
     * @return Response
     */
    public function update(BookingResourceRequest $request, Booking $model)
    {
        try {
            $request = $request->all();
            $model = BookingAction::run('update', $model, $request);
            $data = new BookingResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('offplan::booking.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('offplan/booking/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/booking/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the booking.
     *
     * @param Model   $booking
     *
     * @return Response
     */
    public function destroy(BookingResourceRequest $request, Booking $model)
    {
        try {

            $request = $request->all();
            $model = BookingAction::run('destroy', $model, $request);
            $data = new BookingResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('offplan::booking.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('offplan/booking/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/booking/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
