<?php

namespace Bixo\Offplan\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Offplan\Forms\Floorplan as FloorplanForm;
use Bixo\Offplan\Http\Requests\FloorplanResourceRequest;
use Bixo\Offplan\Http\Resources\FloorplanResource;
use Bixo\Offplan\Http\Resources\FloorplansCollection;
use Bixo\Offplan\Models\Floorplan;
use Bixo\Offplan\Actions\FloorplanAction;
use Bixo\Offplan\Actions\FloorplanActions;


/**
 * Resource controller class for floorplan.
 */
class FloorplanResourceController extends BaseController
{

    /**
     * Initialize floorplan resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = FloorplanForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.offplan.modules'), 'offplan', guard_url('offplan'));
            return $next($request);
        });
    }

    /**
     * Display a list of floorplan.
     *
     * @return Response
     */
    public function index(FloorplanResourceRequest $request)
    {
        $request = $request->all();
        $page = FloorplanActions::run('paginate', $request);

        $data = new FloorplansCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('offplan::floorplan.names'))
            ->view('offplan::floorplan.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function show(FloorplanResourceRequest $request, Floorplan $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new FloorplanResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('offplan::floorplan.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('offplan::floorplan.show')
            ->output();
    }

    /**
     * Show the form for creating a new floorplan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FloorplanResourceRequest $request, Floorplan $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new FloorplanResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('offplan::floorplan.name'))
            ->view('offplan::floorplan.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new floorplan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FloorplanResourceRequest $request, Floorplan $model)
    {
        try {
            $request = $request->all();
            $model = FloorplanAction::run('store', $model, $request);
            $data = new FloorplanResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('offplan::floorplan.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('offplan/floorplan/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/offplan/floorplan'))
                ->redirect();
        }

    }

    /**
     * Show floorplan for editing.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function edit(FloorplanResourceRequest $request, Floorplan $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new FloorplanResource($model);
        // return view('offplan::floorplan.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('offplan::floorplan.name'))
            ->view('offplan::floorplan.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function update(FloorplanResourceRequest $request, Floorplan $model)
    {
        try {
            $request = $request->all();
            $model = FloorplanAction::run('update', $model, $request);
            $data = new FloorplanResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('offplan::floorplan.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('offplan/floorplan/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/floorplan/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the floorplan.
     *
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function destroy(FloorplanResourceRequest $request, Floorplan $model)
    {
        try {

            $request = $request->all();
            $model = FloorplanAction::run('destroy', $model, $request);
            $data = new FloorplanResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('offplan::floorplan.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('offplan/floorplan/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/floorplan/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
